/*****************************************************************************
 *
 *  Copyright (C) 2003 Cdric Brgardis <cedric.bregardis@free.fr>
 *
 *  This file is part of BRIQUOLO
 *
 *  BRIQUOLO is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  BRIQUOLO is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with BRIQUOLO; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#include "EditeurTableau.h"
#include "Gestionnaire.h"

EditeurTableau::EditeurTableau(MOGL_Fenetre * p_Fenetre)
  : TableauBase(p_Fenetre), _PasX(5), _PasY(5), _ElementCourant(NULL), _Ajout(true)
{
  _Curseur=new Curseur(this);
  _Univers->AjouterElement(_Curseur);

  _Camera=new MOGL_Camera();
  _Camera->SetFov(45);
  _Camera->SetHaut(0,0,1);
  _Camera->SetPosition(20,-13, 40);
  _Camera->SetDirection(0,1,-0.8f);
  _Camera->SetClipping(0.2f,2000);

  _Univers->SetCamera(_Camera);

  _PanneauOutilsEditeur=new PanneauOutilsEditeur(_Fenetre);
  _PanneauOutilsEditeur->SetActivation(true);
  _PanneauOutilsEditeur->_ChoixAction->ChangementElement.Connecter(this, &EditeurTableau::_ChangementAction);
}

EditeurTableau::~EditeurTableau()
{
  delete _Curseur;
  delete _Camera;
  delete _PanneauOutilsEditeur;
}

void EditeurTableau::Activer(bool p_Activation)
{
  if (p_Activation!=_Activation)
  {
    if (p_Activation)
    {
      _Fenetre->AjouterInterface(_PanneauOutilsEditeur);
    }
    else
    {
      _Fenetre->RetirerInterface(_PanneauOutilsEditeur);
    }
  }
  TableauBase::Activer(p_Activation);
}

void EditeurTableau::ClavierUp(SDL_keysym * key)
{
  if (!_Pause)
  {
    if (_Ajout)
    {
      switch (key->sym)
      {
        case SDLK_i :
        {
          float x=_Curseur->GetPositionX();
          float y=_Curseur->GetPositionY() + _PanneauOutilsEditeur->GetVariationY();
          if (y>_YMax)
          {
            y=_Curseur->GetPositionY();
          }
          _Curseur->SetPosition(x, y);
          break;
        }
        case SDLK_k :
        {
          float x=_Curseur->GetPositionX();
          float y=_Curseur->GetPositionY() - _PanneauOutilsEditeur->GetVariationY();
          if (y<_YMin)
          {
            y=_Curseur->GetPositionY();
          }
          _Curseur->SetPosition(x, y);
          break;
        }
        case SDLK_j :
        {
          float x=_Curseur->GetPositionX() - _PanneauOutilsEditeur->GetVariationX();
          float y=_Curseur->GetPositionY();
          if (x<_XMin)
          {
            x=_Curseur->GetPositionX();
          }
          _Curseur->SetPosition(x, y);
          break;
        }
        case SDLK_l :
        {
          float x=_Curseur->GetPositionX() + _PanneauOutilsEditeur->GetVariationX();
          float y=_Curseur->GetPositionY();
          if (x>_XMax)
          {
            x=_Curseur->GetPositionX();
          }
          _Curseur->SetPosition(x, y);
          break;
        }
        case SDLK_SPACE :
        {
          ElementTableau * elem=Gestionnaire::GetElementTableau(_PanneauOutilsEditeur->GetIdentifiantElement(), this);
          elem->SetPosition(_Curseur->GetPositionX(), _Curseur->GetPositionY());
          AjouterElementTableau(elem);
          break;
        }
        default:
          break;
      }
    }
    else
    {
      switch (key->sym)
      {
        case SDLK_i :
        {
          ElementTableau * elemTemp=NULL;
          float y=_YMax+1;
          float x=_Curseur->GetPositionX();
          float dist=sqrt((_YMax-_YMin)*(_YMax-_YMin) + (_XMax-_XMin)*(_XMax-_XMin)) +1 ;
          for(Set_ElementTableau::iterator it=_SetElementTableau.begin(); it!=_SetElementTableau.end(); it++)
          {
            float dist2=sqrt((_Curseur->GetPositionY()-(*it)->GetPositionY()) * (_Curseur->GetPositionY()-(*it)->GetPositionY()) +
                             (_Curseur->GetPositionX()-(*it)->GetPositionX()) * (_Curseur->GetPositionX()-(*it)->GetPositionX()));
            if ((*it)->GetPositionY()>_Curseur->GetPositionY() && dist2<dist)
            {
              y=(*it)->GetPositionY();
              x=(*it)->GetPositionX();
              elemTemp=*it;
              dist=dist2;
            }
          }
          if (elemTemp!=NULL)
          {
            _ElementCourant=elemTemp;
            _Curseur->SetPosition(_ElementCourant->GetPositionX(), _ElementCourant->GetPositionY());
          }
          break;
        }
        case SDLK_k :
        {
          ElementTableau * elemTemp=NULL;
          float y=_YMin-1;
          float x=_Curseur->GetPositionX();
          float dist=sqrt((_YMax-_YMin)*(_YMax-_YMin) + (_XMax-_XMin)*(_XMax-_XMin)) +1 ;
          for(Set_ElementTableau::iterator it=_SetElementTableau.begin(); it!=_SetElementTableau.end(); it++)
          {
            float dist2=sqrt((_Curseur->GetPositionY()-(*it)->GetPositionY()) * (_Curseur->GetPositionY()-(*it)->GetPositionY()) +
                             (_Curseur->GetPositionX()-(*it)->GetPositionX()) * (_Curseur->GetPositionX()-(*it)->GetPositionX()));
            if ((*it)->GetPositionY()<_Curseur->GetPositionY() && dist2<dist)
            {
              y=(*it)->GetPositionY();
              x=(*it)->GetPositionX();
              elemTemp=*it;
              dist=dist2;
            }

          }
          if (elemTemp!=NULL)
          {
            _ElementCourant=elemTemp;
            _Curseur->SetPosition(_ElementCourant->GetPositionX(), _ElementCourant->GetPositionY());
          }
          break;
        }
        case SDLK_j :
        {
          ElementTableau * elemTemp=NULL;
          float x=_XMin-1;
          float y=_Curseur->GetPositionY();
          float dist=sqrt((_YMax-_YMin)*(_YMax-_YMin) + (_XMax-_XMin)*(_XMax-_XMin)) +1 ;
          for(Set_ElementTableau::iterator it=_SetElementTableau.begin(); it!=_SetElementTableau.end(); it++)
          {
            float dist2=sqrt((_Curseur->GetPositionY()-(*it)->GetPositionY()) * (_Curseur->GetPositionY()-(*it)->GetPositionY()) +
                             (_Curseur->GetPositionX()-(*it)->GetPositionX()) * (_Curseur->GetPositionX()-(*it)->GetPositionX()));
            if ((*it)->GetPositionX()<_Curseur->GetPositionX() && dist2<dist)
            {
              y=(*it)->GetPositionY();
              x=(*it)->GetPositionX();
              elemTemp=*it;
              dist=dist2;
            }
          }
          if (elemTemp!=NULL)
          {
            _ElementCourant=elemTemp;
            _Curseur->SetPosition(_ElementCourant->GetPositionX(), _ElementCourant->GetPositionY());
          }
          break;
        }
        case SDLK_l :
        {
          ElementTableau * elemTemp=NULL;
          float x=_XMax+1;
          float y=_Curseur->GetPositionY();
          float dist=sqrt((_YMax-_YMin)*(_YMax-_YMin) + (_XMax-_XMin)*(_XMax-_XMin)) +1 ;
          for(Set_ElementTableau::iterator it=_SetElementTableau.begin(); it!=_SetElementTableau.end(); it++)
          {
            float dist2=sqrt((_Curseur->GetPositionY()-(*it)->GetPositionY()) * (_Curseur->GetPositionY()-(*it)->GetPositionY()) +
                             (_Curseur->GetPositionX()-(*it)->GetPositionX()) * (_Curseur->GetPositionX()-(*it)->GetPositionX()));
            if ((*it)->GetPositionX()>_Curseur->GetPositionX() && dist2<dist)
            {
              y=(*it)->GetPositionY();
              x=(*it)->GetPositionX();
              elemTemp=*it;
              dist=dist2;
            }
          }
          if (elemTemp!=NULL)
          {
            _ElementCourant=elemTemp;
            _Curseur->SetPosition(_ElementCourant->GetPositionX(), _ElementCourant->GetPositionY());
          }
          break;
        }
        case SDLK_SPACE :
        {
          if (_ElementCourant!=NULL)
          {
            RetirerElementTableau(_ElementCourant);
            delete _ElementCourant;
            _ElementCourant=NULL;
          }
          break;
        }
        default:
          break;
      }
    }
  }
}

void EditeurTableau::_ChangementAction(unsigned int p_Action)
{
  switch(p_Action)
  {
    case 0:
    {
      _ElementCourant=NULL;
      _Ajout=true;
      break;
    }
    case 1:
    {
      if (_SetElementTableau.size()>0)
      {
        _ElementCourant=*(_SetElementTableau.begin());
        _Curseur->SetPosition(_ElementCourant->GetPositionX(), _ElementCourant->GetPositionY());
      }
      else
      {
        _ElementCourant=NULL;
      }
      _Ajout=false;
      break;
    }
  }
}

void EditeurTableau::SetPause(bool p_Pause)
{
  TableauBase::SetPause(p_Pause);
  _PanneauOutilsEditeur->SetPause(p_Pause);
}
